/* This file is part of the KDE libraries
   SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
   SPDX-FileCopyrightText: 2011 Mario Bensi <mbensi@ipsquad.net>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/
#ifndef __kcompressiondevice_p_h
#define __kcompressiondevice_p_h

#define BUFFER_SIZE 8 * 1024
#define SEEK_BUFFER_SIZE 3 * BUFFER_SIZE

#endif
